@@ -5,7 +5,7 @@
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
 
-import sys, StringIO, string
+import sys, StringIO, string, re
 
 import sqlalchemy.util as util
 import sqlalchemy.sql as sql
@@ -38,12 +38,22 @@ class OracleDateTime(sqltypes.DateTime):
 # Oracle does not allow milliseconds in DATE
 # Oracle does not support TIME columns
 
+# only if cx_oracle contains TIMESTAMP
+class OracleTimestamp(sqltypes.DateTime):
+    def get_col_spec(self):
+        return "TIMESTAMP"
+    def get_dbapi_type(self, dialect):
+        return dialect.TIMESTAMP
+        
 class OracleText(sqltypes.TEXT):
     def get_col_spec(self):
         return "CLOB"
 class OracleString(sqltypes.String):
     def get_col_spec(self):
         return "VARCHAR(%(length)s)" % {'length' : self.length}
+class OracleRaw(sqltypes.Binary):
+    def get_col_spec(self):
+        return "RAW(%(length)s)" % {'length' : self.length}
 class OracleChar(sqltypes.CHAR):
     def get_col_spec(self):
         return "CHAR(%(length)s)" % {'length' : self.length}
@@ -79,17 +89,21 @@ colspecs = {
     sqltypes.Binary : OracleBinary,
     sqltypes.Boolean : OracleBoolean,
     sqltypes.TEXT : OracleText,
+    sqltypes.TIMESTAMP : OracleTimestamp,
     sqltypes.CHAR: OracleChar,
 }
 
-
 ischema_names = {
     'VARCHAR2' : OracleString,
     'DATE' : OracleDateTime,
     'DATETIME' : OracleDateTime,
     'NUMBER' : OracleNumeric,
     'BLOB' : OracleBinary,
-    'CLOB' : OracleText
+    'CLOB' : OracleText,
+    'TIMESTAMP' : OracleTimestamp,
+    'RAW' : OracleRaw,
+    'FLOAT' : OracleNumeric,
+    'DOUBLE PRECISION' : OracleNumeric,
 }
 
 constraintSQL = """SELECT
@@ -97,7 +111,8 @@ constraintSQL = """SELECT
   ac.constraint_type,
   LOWER(loc.column_name) AS local_column,
   LOWER(rem.table_name) AS remote_table,
-  LOWER(rem.column_name) AS remote_column
+  LOWER(rem.column_name) AS remote_column,
+  LOWER(rem.owner) AS remote_owner
 FROM all_constraints ac,
   all_cons_columns loc,
   all_cons_columns rem
@@ -122,16 +137,21 @@ def descriptor():
     ]}
 
 class OracleExecutionContext(default.DefaultExecutionContext):
-    pass
-    
+    def pre_exec(self, engine, proxy, compiled, parameters):
+        super(OracleExecutionContext, self).pre_exec(engine, proxy, compiled, parameters)
+        if self.dialect.auto_setinputsizes:
+                self.set_input_sizes(proxy(), parameters)
+        
 class OracleDialect(ansisql.ANSIDialect):
-    def __init__(self, use_ansi=True, module=None, threaded=True, **kwargs):
+    def __init__(self, use_ansi=True, auto_setinputsizes=False, module=None, threaded=True, **kwargs):
         self.use_ansi = use_ansi
         self.threaded = threaded
         if module is None:
             self.module = cx_Oracle
         else:
             self.module = module
+        self.supports_timestamp = hasattr(self.module, 'TIMESTAMP' )
+        self.auto_setinputsizes = auto_setinputsizes
         ansisql.ANSIDialect.__init__(self, **kwargs)
 
     def dbapi(self):
@@ -180,9 +200,18 @@ class OracleDialect(ansisql.ANSIDialect):
     def has_table(self, connection, table_name):
         cursor = connection.execute("""select table_name from all_tables where table_name=:name""", {'name':table_name.upper()})
         return bool( cursor.fetchone() is not None )
+
+    def has_sequence(self, connection, sequence_name):
+        cursor = connection.execute("""select sequence_name from all_sequences where sequence_name=:name""", {'name':sequence_name.upper()})
+        return bool( cursor.fetchone() is not None )
         
     def reflecttable(self, connection, table):
-        c = connection.execute ("select distinct OWNER from ALL_TAB_COLUMNS where TABLE_NAME = :table_name", {'table_name':table.name.upper()})
+        preparer = self.identifier_preparer
+        if not preparer.should_quote(table):
+            name = table.name.upper()
+        else:
+            name = table.name
+        c = connection.execute ("select distinct OWNER from ALL_TAB_COLUMNS where TABLE_NAME = :table_name", {'table_name':name})
         rows = c.fetchall()
         if not rows :
             raise exceptions.NoSuchTableError(table.name)
@@ -198,7 +227,7 @@ class OracleDialect(ansisql.ANSIDialect):
                 else:
                     raise exceptions.AssertionError("There are multiple tables with name %s in the schema, you must specifie owner"%table.name)
 
-        c = connection.execute ("select COLUMN_NAME, DATA_TYPE, DATA_LENGTH, DATA_PRECISION, DATA_SCALE, NULLABLE, DATA_DEFAULT from ALL_TAB_COLUMNS where TABLE_NAME = :table_name and OWNER = :owner", {'table_name':table.name.upper(), 'owner':owner})
+        c = connection.execute ("select COLUMN_NAME, DATA_TYPE, DATA_LENGTH, DATA_PRECISION, DATA_SCALE, NULLABLE, DATA_DEFAULT from ALL_TAB_COLUMNS where TABLE_NAME = :table_name and OWNER = :owner", {'table_name':name, 'owner':owner})
         
         while True:
             row = c.fetchone()
@@ -224,15 +253,21 @@ class OracleDialect(ansisql.ANSIDialect):
             elif coltype=='CHAR' or coltype=='VARCHAR2':
                 coltype = ischema_names.get(coltype, OracleString)(length)
             else:
-                coltype = ischema_names.get(coltype)
+                coltype = re.sub(r'\(\d+\)', '', coltype)
+                try:
+                    coltype = ischema_names[coltype]
+                except KeyError:
+                    raise exceptions.AssertionError("Cant get coltype for type '%s'" % coltype)
                
             colargs = []
             if default is not None:
                 colargs.append(schema.PassiveDefault(sql.text(default)))
+          
+            # if name comes back as all upper, assume its case folded 
+            if (name.upper() == name): 
+                name = name.lower()
             
-            name = name.lower()
-            
-            table.append_item (schema.Column(name, coltype, nullable=nullable, *colargs))
+            table.append_column(schema.Column(name, coltype, nullable=nullable, *colargs))
 
        
         c = connection.execute(constraintSQL, {'table_name' : table.name.upper(), 'owner' : owner})
@@ -242,9 +277,9 @@ class OracleDialect(ansisql.ANSIDialect):
             if row is None:
                 break
             #print "ROW:" , row                
-            (cons_name, cons_type, local_column, remote_table, remote_column) = row
+            (cons_name, cons_type, local_column, remote_table, remote_column, remote_owner) = row
             if cons_type == 'P':
-                table.c[local_column]._set_primary_key()
+                table.primary_key.add(table.c[local_column])
             elif cons_type == 'R':
                 try:
                     fk = fks[cons_name]
@@ -252,14 +287,14 @@ class OracleDialect(ansisql.ANSIDialect):
                    fk = ([], [])
                    fks[cons_name] = fk
                 refspec = ".".join([remote_table, remote_column])
-                schema.Table(remote_table, table.metadata, autoload=True, autoload_with=connection)
+                schema.Table(remote_table, table.metadata, autoload=True, autoload_with=connection, owner=remote_owner)
                 if local_column not in fk[0]:
                     fk[0].append(local_column)
                 if refspec not in fk[1]:
                     fk[1].append(refspec)
 
         for name, value in fks.iteritems():
-            table.append_item(schema.ForeignKeyConstraint(value[0], value[1], name=name))
+            table.append_constraint(schema.ForeignKeyConstraint(value[0], value[1], name=name))
 
     def do_executemany(self, c, statement, parameters, context=None):
         rowcount = 0
@@ -287,6 +322,7 @@ class OracleCompiler(ansisql.ANSICompiler):
         
         self.froms[join] = self.get_from_text(join.left) + ", " + self.get_from_text(join.right)
         self.wheres[join] = sql.and_(self.wheres.get(join.left, None), join.onclause)
+        self.strings[join] = self.froms[join]
 
         if join.isouter:
             # if outer join, push on the right side table as the current "outertable"
@@ -320,6 +356,40 @@ class OracleCompiler(ansisql.ANSICompiler):
                 self.parameters[c.key] = None
         return ansisql.ANSICompiler.visit_insert(self, insert)
 
+    def _TODO_visit_compound_select(self, select):
+        """need to determine how to get LIMIT/OFFSET into a UNION for oracle"""
+        if getattr(select, '_oracle_visit', False):
+            # cancel out the compiled order_by on the select
+            if hasattr(select, "order_by_clause"):
+                self.strings[select.order_by_clause] = ""
+            ansisql.ANSICompiler.visit_compound_select(self, select)
+            return
+            
+        if select.limit is not None or select.offset is not None:
+            select._oracle_visit = True
+            # to use ROW_NUMBER(), an ORDER BY is required. 
+            orderby = self.strings[select.order_by_clause]
+            if not orderby:
+                orderby = select.oid_column
+                orderby.accept_visitor(self)
+                orderby = self.strings[orderby]
+            class SelectVisitor(sql.ClauseVisitor):
+                def visit_select(self, select):
+                    select.append_column(sql.column("ROW_NUMBER() OVER (ORDER BY %s)" % orderby).label("ora_rn"))
+            select.accept_visitor(SelectVisitor())
+            limitselect = sql.select([c for c in select.c if c.key!='ora_rn'])
+            if select.offset is not None:
+                limitselect.append_whereclause("ora_rn>%d" % select.offset)
+                if select.limit is not None:
+                    limitselect.append_whereclause("ora_rn<=%d" % (select.limit + select.offset))
+            else:
+                limitselect.append_whereclause("ora_rn<=%d" % select.limit)
+            limitselect.accept_visitor(self)
+            self.strings[select] = self.strings[limitselect]
+            self.froms[select] = self.froms[limitselect]
+        else:
+            ansisql.ANSICompiler.visit_compound_select(self, select)
+        
     def visit_select(self, select):
         """looks for LIMIT and OFFSET in a select statement, and if so tries to wrap it in a 
         subquery with row_number() criterion."""
@@ -340,7 +410,7 @@ class OracleCompiler(ansisql.ANSICompiler):
                 orderby = select.oid_column
                 orderby.accept_visitor(self)
                 orderby = self.strings[orderby]
-            select.append_column(sql.ColumnClause("ROW_NUMBER() OVER (ORDER BY %s)" % orderby).label("ora_rn"))
+            select.append_column(sql.column("ROW_NUMBER() OVER (ORDER BY %s)" % orderby).label("ora_rn"))
             limitselect = sql.select([c for c in select.c if c.key!='ora_rn'])
             if select.offset is not None:
                 limitselect.append_whereclause("ora_rn>%d" % select.offset)
@@ -357,9 +427,15 @@ class OracleCompiler(ansisql.ANSICompiler):
     def limit_clause(self, select):
         return ""
 
+    def for_update_clause(self, select):
+        if select.for_update=="nowait":
+            return " FOR UPDATE NOWAIT"
+        else:
+            return super(OracleCompiler, self).for_update_clause(select)
+
 class OracleSchemaGenerator(ansisql.ANSISchemaGenerator):
     def get_column_specification(self, column, **kwargs):
-        colspec = column.name
+        colspec = self.preparer.format_column(column)
         colspec += " " + column.type.engine_impl(self.engine).get_col_spec()
         default = self.get_column_default_string(column)
         if default is not None:
@@ -370,13 +446,15 @@ class OracleSchemaGenerator(ansisql.ANSISchemaGenerator):
         return colspec
 
     def visit_sequence(self, sequence):
-        self.append("CREATE SEQUENCE %s" % sequence.name)
-        self.execute()
+        if not self.engine.dialect.has_sequence(self.connection, sequence.name):
+            self.append("CREATE SEQUENCE %s" % self.preparer.format_sequence(sequence))
+            self.execute()
 
 class OracleSchemaDropper(ansisql.ANSISchemaDropper):
     def visit_sequence(self, sequence):
-        self.append("DROP SEQUENCE %s" % sequence.name)
-        self.execute()
+        if self.engine.dialect.has_sequence(self.connection, sequence.name):
+            self.append("DROP SEQUENCE %s" % sequence.name)
+            self.execute()
 
 class OracleDefaultRunner(ansisql.ANSIDefaultRunner):
     def exec_default_sql(self, default):
