@@ -299,7 +299,7 @@ class Connection(Connectable):
     # poor man's multimethod/generic function thingy
     executors = {
         sql.ClauseElement : execute_clauseelement,
-        sql.Compiled : execute_compiled,
+        sql.ClauseVisitor : execute_compiled,
         schema.SchemaItem:execute_default,
         str.__mro__[-2] : execute_text
     }
@@ -396,7 +396,7 @@ class Engine(sql.Executor, Connectable):
     Connects a ConnectionProvider, a Dialect and a CompilerFactory together to 
     provide a default implementation of SchemaEngine.
     """
-    def __init__(self, connection_provider, dialect, echo=None, **kwargs):
+    def __init__(self, connection_provider, dialect, echo=None):
         self.connection_provider = connection_provider
         self.dialect=dialect
         self.echo = echo
@@ -594,7 +594,7 @@ class ResultProxy(object):
             self.cursor.close()
             if self.connection.should_close_with_result and self.dialect.supports_autoclose_results:
                 self.connection.close()
-    
+
     def _convert_key(self, key):
         """given a key, which could be a ColumnElement, string, etc., matches it to the 
         appropriate key we got from the result set's metadata; then cache it locally for quick re-access."""
@@ -624,7 +624,7 @@ class ResultProxy(object):
         try:
             self._convert_key(key)
             return True
-        except exceptions.NoSuchColumnError:
+        except KeyError:
             return False
         
     def _get_col(self, row, key):
