@@ -8,19 +8,23 @@ to declare table or mapper classes ahead of time.
 Suppose we have a database with users, books, and loans tables
 (corresponding to the PyWebOff dataset, if you're curious).
 For testing purposes, we'll create this db as follows:
+
     >>> from sqlalchemy import create_engine
     >>> e = create_engine('sqlite:///:memory:')
     >>> for sql in _testsql: e.execute(sql) #doctest: +ELLIPSIS
     <...
 
 Creating a SqlSoup gateway is just like creating an SqlAlchemy engine:
+
     >>> from sqlalchemy.ext.sqlsoup import SqlSoup
     >>> db = SqlSoup('sqlite:///:memory:')
 
 or, you can re-use an existing metadata:
+
     >>> db = SqlSoup(BoundMetaData(e))
 
 You can optionally specify a schema within the database for your SqlSoup:
+
     # >>> db.schema = myschemaname
 
 
@@ -28,28 +32,33 @@ Loading objects
 ===============
 
 Loading objects is as easy as this:
+
     >>> users = db.users.select()
     >>> users.sort()
     >>> users
     [MappedUsers(name='Joe Student',email='student@example.edu',password='student',classname=None,admin=0), MappedUsers(name='Bhargan Basepair',email='basepair@example.edu',password='basepair',classname=None,admin=1)]
 
 Of course, letting the database do the sort is better (".c" is short for ".columns"):
+
     >>> db.users.select(order_by=[db.users.c.name])
     [MappedUsers(name='Bhargan Basepair',email='basepair@example.edu',password='basepair',classname=None,admin=1), MappedUsers(name='Joe Student',email='student@example.edu',password='student',classname=None,admin=0)]
 
 Field access is intuitive:
+
     >>> users[0].email
     u'student@example.edu'
 
 Of course, you don't want to load all users very often.  Let's add a WHERE clause.
 Let's also switch the order_by to DESC while we're at it.
+
     >>> from sqlalchemy import or_, and_, desc
-	>>> where = or_(db.users.c.name=='Bhargan Basepair', db.users.c.email=='student@example.edu')
+    >>> where = or_(db.users.c.name=='Bhargan Basepair', db.users.c.email=='student@example.edu')
     >>> db.users.select(where, order_by=[desc(db.users.c.name)])
     [MappedUsers(name='Joe Student',email='student@example.edu',password='student',classname=None,admin=0), MappedUsers(name='Bhargan Basepair',email='basepair@example.edu',password='basepair',classname=None,admin=1)]
 
 You can also use the select...by methods if you're querying on a single column.
 This allows using keyword arguments as column names:
+
     >>> db.users.selectone_by(name='Bhargan Basepair')
     MappedUsers(name='Bhargan Basepair',email='basepair@example.edu',password='basepair',classname=None,admin=1)
 
@@ -61,13 +70,14 @@ All the SqlAlchemy Query select variants are available.
 Here's a quick summary of these methods:
 
 - get(PK): load a single object identified by its primary key (either a scalar, or a tuple)
-- select(Clause, **kwargs): perform a select restricted by the Clause argument; returns a list of objects.  The most common clause argument takes the form "db.tablename.c.columname == value."  The most common optional argument is order_by.
-- select_by(**params): the *_by selects allow using bare column names.  (columname=value)This feels more natural to most Python programmers; the downside is you can't specify order_by or other select options.
+- select(Clause, \*\*kwargs): perform a select restricted by the Clause argument; returns a list of objects.  The most common clause argument takes the form "db.tablename.c.columname == value."  The most common optional argument is order_by.
+- select_by(\*\*params): select methods ending with _by allow using bare column names.  (columname=value)  This feels more natural to most Python programmers; the downside is you can't specify order_by or other select options.
 - selectfirst, selectfirst_by: returns only the first object found; equivalent to select(...)[0] or select_by(...)[0], except None is returned if no rows are selected.
 - selectone, selectone_by: like selectfirst or selectfirst_by, but raises if less or more than one object is selected.
 - count, count_by: returns an integer count of the rows selected.
 
 See the SqlAlchemy documentation for details:
+
 - http://www.sqlalchemy.org/docs/datamapping.myt#datamapping_query for general info and examples,
 - http://www.sqlalchemy.org/docs/sqlconstruction.myt for details on constructing WHERE clauses.
 
@@ -76,6 +86,7 @@ Modifying objects
 =================
 
 Modifying objects is intuitive:
+
     >>> user = _
     >>> user.email = 'basepair+nospam@example.edu'
     >>> db.flush()
@@ -84,8 +95,8 @@ Modifying objects is intuitive:
 multiple updates to a single object will be turned into a single UPDATE
 statement when you flush.)
 
-
 To finish covering the basics, let's insert a new loan, then delete it:
+
     >>> book_id = db.books.selectfirst(db.books.c.title=='Regional Variation in Moss').id
     >>> db.loans.insert(book_id=book_id, user_name=user.name)
     MappedLoans(book_id=2,user_name='Bhargan Basepair',loan_date=None)
@@ -100,6 +111,7 @@ this time using the loans table's delete method.  (For SQLAlchemy experts:
 note that no flush() call is required since this
 delete acts at the SQL level, not at the Mapper level.)  The same where-clause construction rules
 apply here as to the select methods.
+
     >>> db.loans.insert(book_id=book_id, user_name=user.name)
     MappedLoans(book_id=2,user_name='Bhargan Basepair',loan_date=None)
     >>> db.flush()
@@ -121,28 +133,58 @@ more efficient to have the database perform the necessary join.  (Here
 we do not have "a lot of data," but hopefully the concept is still clear.)
 SQLAlchemy is smart enough to recognize that loans has a foreign key
 to users, and uses that as the join condition automatically.
+
     >>> join1 = db.join(db.users, db.loans, isouter=True)
     >>> join1.select_by(name='Joe Student')
     [MappedJoin(name='Joe Student',email='student@example.edu',password='student',classname=None,admin=0,book_id=1,user_name='Joe Student',loan_date=datetime.datetime(2006, 7, 12, 0, 0))]
 
+If you're unfortunate enough to be using MySQL with the default MyISAM
+storage engine, you'll have to specify the join condition manually,
+since MyISAM does not store foreign keys.  Here's the same join again,
+with the join condition explicitly specified:
+
+    >>> db.join(db.users, db.loans, db.users.c.name==db.loans.c.user_name, isouter=True)
+    <class 'sqlalchemy.ext.sqlsoup.MappedJoin'>
+
 You can compose arbitrarily complex joins by combining Join objects with
-tables or other joins.
+tables or other joins.  Here we combine our first join with the books table:
+
     >>> join2 = db.join(join1, db.books)
     >>> join2.select()
     [MappedJoin(name='Joe Student',email='student@example.edu',password='student',classname=None,admin=0,book_id=1,user_name='Joe Student',loan_date=datetime.datetime(2006, 7, 12, 0, 0),id=1,title='Mustards I Have Known',published_year='1989',authors='Jones')]
 
 If you join tables that have an identical column name, wrap your join with "with_labels",
-and all the columns will be prefixed with their table name:
-    >>> db.with_labels(join1).select()
-    [MappedUsersLoansJoin(users_name='Joe Student',users_email='student@example.edu',users_password='student',users_classname=None,users_admin=0,loans_book_id=1,loans_user_name='Joe Student',loans_loan_date=datetime.datetime(2006, 7, 12, 0, 0))]
+to disambiguate columns with their table name:
+
+    >>> db.with_labels(join1).c.keys()
+    ['users_name', 'users_email', 'users_password', 'users_classname', 'users_admin', 'loans_book_id', 'loans_user_name', 'loans_loan_date']
 
 
 Advanced Use
 ============
 
-You can access the SqlSoup's engine attribute to compose SQL directly.
-The engine's <b>execute</b> method corresponds
-to the one of a DBAPI cursor, and returns a ResultProxy that has <b>fetch</b> methods
+Mapping arbitrary Selectables
+-----------------------------
+
+SqlSoup can map any SQLAlchemy Selectable with the map method.  Let's map a Select object that uses an aggregate function; we'll use the SQLAlchemy Table that SqlSoup introspected as the basis.  (Since we're not mapping to a simple table or join, we need to tell SQLAlchemy how to find the "primary key," which just needs to be unique within the select, and not necessarily correspond to a "real" PK in the database.)
+
+    >>> from sqlalchemy import select, func
+    >>> b = db.books._table
+    >>> s = select([b.c.published_year, func.count('*').label('n')], from_obj=[b], group_by=[b.c.published_year])
+    >>> s = s.alias('years_with_count')
+    >>> years_with_count = db.map(s, primary_key=[s.c.published_year])
+    >>> years_with_count.select_by(published_year='1989')
+    [MappedBooks(published_year='1989',n=1)]
+    
+Obviously if we just wanted to get a list of counts associated with book years once, raw SQL is going to be less work.  The advantage of mapping a Select is reusability, both standalone and in Joins.  (And if you go to full SQLAlchemy, you can perform mappings like this directly to your object models.)
+
+
+Raw SQL
+-------
+
+You can access the SqlSoup's ``engine`` attribute to compose SQL directly.
+The engine's ``execute`` method corresponds
+to the one of a DBAPI cursor, and returns a ``ResultProxy`` that has ``fetch`` methods
 you would also see on a cursor.
 
     >>> rp = db.engine.execute('select name, email from users order by name')
@@ -170,6 +212,11 @@ Boring tests here.  Nothing of real expository value.
     Traceback (most recent call last):
     ...
     NoSuchTableError: nosuchtable
+
+    >>> years_with_count.insert(published_year='2007', n=1)
+    Traceback (most recent call last):
+    ...
+    InvalidRequestError: SQLSoup can only modify mapped Tables (found: Alias)
 """
 
 from sqlalchemy import *
@@ -240,17 +287,25 @@ class PKNotFoundError(SQLAlchemyError): pass
 
 # metaclass is necessary to expose class methods with getattr, e.g.
 # we want to pass db.users.select through to users._mapper.select
+def _ddl_check(cls):
+    if not isinstance(cls._table, Table):
+        msg = 'SQLSoup can only modify mapped Tables (found: %s)' \
+              % cls._table.__class__.__name__
+        raise InvalidRequestError(msg)
 class TableClassType(type):
     def insert(cls, **kwargs):
+        _ddl_check(cls)
         o = cls()
         o.__dict__.update(kwargs)
         return o
-    def _selectable(cls):
-        return cls._table
     def delete(cls, *args, **kwargs):
+        _ddl_check(cls)
         cls._table.delete(*args, **kwargs).execute()
     def update(cls, whereclause=None, values=None, **kwargs):
+        _ddl_check(cls)
         cls._table.update(whereclause, values).execute(**kwargs)
+    def _selectable(cls):
+        return cls._table
     def __getattr__(cls, attr):
         if attr == '_query':
             # called during mapper init
@@ -269,7 +324,7 @@ def _selectable_name(selectable):
     if isinstance(selectable, sql.Alias):
         return _selectable_name(selectable.selectable)
     elif isinstance(selectable, sql.Select):
-        return ''.join([_selectable_name(s) for s in selectable._froms])
+        return ''.join([_selectable_name(s) for s in selectable.froms])
     elif isinstance(selectable, schema.Table):
         return selectable.name.capitalize()
     else:
@@ -278,7 +333,7 @@ def _selectable_name(selectable):
             x = x[1:]
         return x
 
-def class_for_table(selectable):
+def class_for_table(selectable, **mapper_kwargs):
     if not hasattr(selectable, '_selectable') \
     or selectable._selectable() != selectable:
         raise 'class_for_table requires a selectable as its argument'
@@ -309,7 +364,8 @@ def class_for_table(selectable):
     klass._mapper = mapper(klass,
                            selectable,
                            extension=objectstore.mapper_extension,
-                           allow_null_pks=_is_outer_join(selectable))
+                           allow_null_pks=_is_outer_join(selectable),
+                           **mapper_kwargs)
     klass._query = Query(klass._mapper)
     return klass
 
@@ -339,19 +395,19 @@ class SqlSoup:
         objectstore.get_session().flush()
     def rollback(self):
         objectstore.clear()
-    def _map(self, selectable):
+    def map(self, selectable, **kwargs):
         try:
             t = self._cache[selectable]
         except KeyError:
-            t = class_for_table(selectable)
+            t = class_for_table(selectable, **kwargs)
             self._cache[selectable] = t
         return t
     def with_labels(self, item):
         # TODO give meaningful aliases
-        return self._map(item._selectable().select(use_labels=True).alias('foo'))
+        return self.map(item._selectable().select(use_labels=True).alias('foo'))
     def join(self, *args, **kwargs):
         j = join(*args, **kwargs)
-        return self._map(j)
+        return self.map(j)
     def __getattr__(self, attr):
         try:
             t = self._cache[attr]
