@@ -15,8 +15,8 @@ dirty, or deleted and provides the capability to flush all those changes at once
 """
 
 from sqlalchemy import util, logging, topological
-import sqlalchemy
-import attributes
+from sqlalchemy.orm import attributes
+from sqlalchemy.orm.mapper import object_mapper, class_mapper
 from sqlalchemy.exceptions import *
 import StringIO
 import weakref
@@ -73,11 +73,14 @@ class UnitOfWork(object):
     """main UOW object which stores lists of dirty/new/deleted objects.  
     provides top-level "flush" functionality as well as the transaction 
     boundaries with the SQLEngine(s) involved in a write operation."""
-    def __init__(self, identity_map=None):
+    def __init__(self, identity_map=None, weak_identity_map=False):
         if identity_map is not None:
             self.identity_map = identity_map
         else:
-            self.identity_map = weakref.WeakValueDictionary()
+            if weak_identity_map:
+                self.identity_map = weakref.WeakValueDictionary()
+            else:
+                self.identity_map = {}
             
         self.new = util.Set() #OrderedSet()
         self.deleted = util.Set()
@@ -101,7 +104,14 @@ class UnitOfWork(object):
         if (hasattr(obj, '_instance_key') and not self.identity_map.has_key(obj._instance_key)) or \
             (not hasattr(obj, '_instance_key') and obj not in self.new):
             raise InvalidRequestError("Instance '%s' is not attached or pending within this session" % repr(obj))
-
+    
+    def _is_valid(self, obj):
+        if (hasattr(obj, '_instance_key') and not self.identity_map.has_key(obj._instance_key)) or \
+            (not hasattr(obj, '_instance_key') and obj not in self.new):
+            return False
+        else:
+            return True
+        
     def register_attribute(self, class_, key, uselist, **kwargs):
         attribute_manager.register_attribute(class_, key, uselist, **kwargs)
 
@@ -109,17 +119,13 @@ class UnitOfWork(object):
         attribute_manager.set_callable(obj, key, func, uselist, **kwargs)
     
     def register_clean(self, obj):
-        try:
+        if obj in self.new:
             self.new.remove(obj)
-        except KeyError:
-            pass
         if not hasattr(obj, '_instance_key'):
             mapper = object_mapper(obj)
             obj._instance_key = mapper.instance_key(obj)
-        try:
+        if hasattr(obj, '_sa_insert_order'):
             delattr(obj, '_sa_insert_order')
-        except AttributeError:
-            pass
         self.identity_map[obj._instance_key] = obj
         attribute_manager.commit(obj)
         
@@ -219,9 +225,9 @@ class UOWTransaction(object):
         registration is entered for the object."""
         #print "REGISTER", repr(obj), repr(getattr(obj, '_instance_key', None)), str(isdelete), str(listonly)
         
-        # things can get really confusing if theres duplicate instances floating around,
-        # so make sure everything is OK
-        self.uow._validate_obj(obj)
+        # if object is not in the overall session, do nothing
+        if not self.uow._is_valid(obj):
+            return
             
         mapper = object_mapper(obj)
         self.mappers.add(mapper)
@@ -286,7 +292,8 @@ class UOWTransaction(object):
         # when the task from "mapper" executes, take the objects from the task corresponding
         # to "mapperfrom"'s list of save/delete objects, and send them to "processor"
         # for dependency processing
-        #print "registerprocessor", str(mapper), repr(processor.key), str(mapperfrom)
+        
+        #print "registerprocessor", str(mapper), repr(processor), repr(processor.key), str(mapperfrom)
         
         # correct for primary mapper (the mapper offcially associated with the class)
         mapper = mapper.primary_mapper()
@@ -589,15 +596,11 @@ class UOWTask(object):
                 for taskelement in task.get_elements(polymorphic=False):
                     obj = taskelement.obj
                     object_to_original_task[obj] = task
-                    #print "OBJ", repr(obj), "TASK", repr(task)
 
                     for dep in deps_by_targettask.get(task, []):
                         # is this dependency involved in one of the cycles ?
-                        #print "DEP iterate", dep.processor.key, dep.processor.parent, dep.processor.mapper
                         if not dependency_in_cycles(dep):
-                            #print "NOT IN CYCLE"
                             continue
-                        #print "DEP", dep.processor.key    
                         (processor, targettask) = (dep.processor, dep.targettask)
                         isdelete = taskelement.isdelete
 
@@ -611,13 +614,26 @@ class UOWTask(object):
                         childlist = childlist.added_items() + childlist.unchanged_items() + childlist.deleted_items()
 
                         for o in childlist:
-                            if o is None or not childtask.contains_object(o, polymorphic=True):
+
+                            # other object is None.  this can occur if the relationship is many-to-one
+                            # or one-to-one, and None was set.  the "removed" object will be picked
+                            # up in this iteration via the deleted_items() part of the collection.
+                            if o is None:
                                 continue
-                            #print "parent/child", obj, o
+
+                            # the other object is not in the UOWTransaction !  but if we are many-to-one,
+                            # we need a task in order to attach dependency operations, so establish a "listonly"
+                            # task
+                            if not childtask.contains_object(o, polymorphic=True):
+                                childtask.append(o, listonly=True)
+                                object_to_original_task[o] = childtask
+
+                            # create a tuple representing the "parent/child"
                             whosdep = dep.whose_dependent_on_who(obj, o)
-                            #print "WHOSEDEP", dep.processor.key, dep.processor.direction, whosdep
                             if whosdep is not None:
+                                # append the tuple to the partial ordering.
                                 tuples.append(whosdep)
+
                                 # create a UOWDependencyProcessor representing this pair of objects.
                                 # append it to a UOWTask
                                 if whosdep[0] is obj:
@@ -637,7 +653,6 @@ class UOWTask(object):
         # create a tree of UOWTasks corresponding to the tree of object instances
         # created by the DependencySorter
         def make_task_tree(node, parenttask, nexttasks):
-            #print "MAKETASKTREE", node.item, parenttask
             originating_task = object_to_original_task[node.item]
             t = nexttasks.get(originating_task, None)
             if t is None:
@@ -843,11 +858,5 @@ class UOWExecutor(object):
 class DependencySorter(topological.QueueDependencySorter):
     pass
 
-def object_mapper(obj):
-    return sqlalchemy.object_mapper(obj)
-
-def class_mapper(class_):
-    return sqlalchemy.class_mapper(class_)
-
 attribute_manager = UOWAttributeManager()
 
