@@ -7,11 +7,9 @@ import testbase
 from tables import *
 import tables
 
-# TODO: need assertion conditions in this suite
+"""test cyclical mapper relationships.  Many of the assertions are provided
+via running with postgres, which is strict about foreign keys."""
 
-
-"""test cyclical mapper relationships.  No assertions yet, but run it with postgres and the 
-foreign key checks alone will usually not work if something is wrong"""
 class Tester(object):
     def __init__(self, data=None):
         self.data = data
@@ -45,7 +43,7 @@ class SelfReferentialTest(AssertMixin):
             pass
         m1 = mapper(C1, t1, properties = {
             'c1s':relation(C1, cascade="all"),
-            'parent':relation(C1, primaryjoin=t1.c.parent_c1==t1.c.c1, foreignkey=t1.c.c1, lazy=True, uselist=False)
+            'parent':relation(C1, primaryjoin=t1.c.parent_c1==t1.c.c1, remote_side=t1.c.c1, lazy=True, uselist=False)
         })
         a = C1('head c1')
         a.c1s.append(C1('another c1'))
@@ -54,6 +52,31 @@ class SelfReferentialTest(AssertMixin):
         sess.flush()
         sess.delete(a)
         sess.flush()
+    
+    def testmanytooneonly(self):
+        """test that the circular dependency sort can assemble a many-to-one dependency processor
+        when only the object on the "many" side is actually in the list of modified objects.
+        this requires that the circular sort add the other side of the relation into the UOWTransaction
+        so that the dependency operation can be tacked onto it.
+        
+        This also affects inheritance relationships since they rely upon circular sort as well.
+        """
+        class C1(Tester):
+            pass
+        mapper(C1, t1, properties={
+            'parent':relation(C1, primaryjoin=t1.c.parent_c1==t1.c.c1, remote_side=t1.c.c1)
+        })
+        sess = create_session()
+        c1 = C1()
+        sess.save(c1)
+        sess.flush()
+        sess.clear()
+        c1 = sess.query(C1).get(c1.c1)
+        c2 = C1()
+        c2.parent = c1
+        sess.save(c2)
+        sess.flush()
+        assert c2.parent_c1==c1.c1
         
     def testcycle(self):
         class C1(Tester):
@@ -95,6 +118,113 @@ class SelfReferentialTest(AssertMixin):
             assert False
         except exceptions.ArgumentError:
             assert True
+
+class SelfReferentialNoPKTest(AssertMixin):
+    def setUpAll(self):
+        global table, meta
+        meta = BoundMetaData(testbase.db)
+        table = Table('item', meta,
+           Column('id', Integer, primary_key=True),
+           Column('uuid', String(32), unique=True, nullable=False),
+           Column('parent_uuid', String(32), ForeignKey('item.uuid'), nullable=True),
+        )
+        meta.create_all()
+    def tearDown(self):
+        table.delete().execute()
+    def tearDownAll(self):
+        meta.drop_all()
+    def testbasic(self):
+        class TT(object):
+            def __init__(self):
+                self.uuid = hex(id(self))
+        mapper(TT, table, properties={'children':relation(TT, remote_side=[table.c.parent_uuid], backref=backref('parent', remote_side=[table.c.uuid]))})
+        s = create_session()
+        t1 = TT()
+        t1.children.append(TT())
+        t1.children.append(TT())
+        s.save(t1)
+        s.flush()
+        s.clear()
+        t = s.query(TT).get_by(id=t1.id)
+        assert t.children[0].parent_uuid == t1.uuid
+    def testlazyclause(self):
+        class TT(object):
+            def __init__(self):
+                self.uuid = hex(id(self))
+        mapper(TT, table, properties={'children':relation(TT, remote_side=[table.c.parent_uuid], backref=backref('parent', remote_side=[table.c.uuid]))})
+        s = create_session()
+        t1 = TT()
+        t2 = TT()
+        t1.children.append(t2)
+        s.save(t1)
+        s.flush()
+        s.clear()
+
+        t = s.query(TT).get_by(id=t2.id)
+        assert t.uuid == t2.uuid
+        assert t.parent.uuid == t1.uuid
+        
+class InheritTestOne(AssertMixin):
+    def setUpAll(self):
+        global parent, child1, child2, meta
+        meta = BoundMetaData(testbase.db)
+        parent = Table("parent", meta,
+            Column("id", Integer, primary_key=True),
+            Column("parent_data", String(50)),
+            Column("type", String(10))
+            )
+
+        child1 = Table("child1", meta,
+            Column("id", Integer, ForeignKey("parent.id"), primary_key=True),
+            Column("child1_data", String(50))
+            )
+
+        child2 = Table("child2", meta,
+            Column("id", Integer, ForeignKey("parent.id"), primary_key=True),
+            Column("child1_id", Integer, ForeignKey("child1.id"), nullable=False),
+            Column("child2_data", String(50))
+            )
+        meta.create_all()
+    def tearDownAll(self):
+        meta.drop_all()
+    def testmanytooneonly(self):
+        """test similar to SelfReferentialTest.testmanytooneonly"""
+        class Parent(object):
+                pass
+
+        mapper(Parent, parent)
+
+        class Child1(Parent):
+                pass
+
+        mapper(Child1, child1, inherits=Parent)
+
+        class Child2(Parent):
+                pass
+
+        mapper(Child2, child2, properties={
+                        "child1": relation(Child1,
+                                primaryjoin=child2.c.child1_id==child1.c.id,
+                        )
+                },inherits=Parent)
+
+        session = create_session()
+
+        c1 = Child1()
+        c1.child1_data = "qwerty"
+        session.save(c1)
+        session.flush()
+        session.clear()
+
+        c1 = session.query(Child1).get_by(child1_data="qwerty")
+        c2 = Child2()
+        c2.child1 = c1
+        c2.child2_data = "asdfgh"
+        session.save(c2)
+        # the flush will fail if the UOW does not set up a many-to-one DP 
+        # attached to a task corresponding to c1, since "child1_id" is not nullable
+        session.flush()
+        
 class BiDirectionalOneToManyTest(AssertMixin):
     """tests two mappers with a one-to-many relation to each other."""
     def setUpAll(self):
@@ -235,8 +365,8 @@ class OneToManyManyToOneTest(AssertMixin):
 
         Ball.mapper = mapper(Ball, ball)
         Person.mapper = mapper(Person, person, properties= dict(
-         balls = relation(Ball.mapper, primaryjoin=ball.c.person_id==person.c.id, foreignkey=ball.c.person_id),
-         favorateBall = relation(Ball.mapper, primaryjoin=person.c.favorite_ball_id==ball.c.id, foreignkey=person.c.favorite_ball_id),
+         balls = relation(Ball.mapper, primaryjoin=ball.c.person_id==person.c.id, remote_side=ball.c.person_id),
+         favorateBall = relation(Ball.mapper, primaryjoin=person.c.favorite_ball_id==ball.c.id, remote_side=person.c.favorite_ball_id),
          )
         )
 
@@ -262,8 +392,8 @@ class OneToManyManyToOneTest(AssertMixin):
 
         Ball.mapper = mapper(Ball, ball)
         Person.mapper = mapper(Person, person, properties= dict(
-         balls = relation(Ball.mapper, primaryjoin=ball.c.person_id==person.c.id, foreignkey=ball.c.person_id, post_update=False, private=True),
-         favorateBall = relation(Ball.mapper, primaryjoin=person.c.favorite_ball_id==ball.c.id, foreignkey=person.c.favorite_ball_id, post_update=True),
+         balls = relation(Ball.mapper, primaryjoin=ball.c.person_id==person.c.id, remote_side=ball.c.person_id, post_update=False, private=True),
+         favorateBall = relation(Ball.mapper, primaryjoin=person.c.favorite_ball_id==ball.c.id, remote_side=person.c.favorite_ball_id, post_update=True),
          )
         )
 
@@ -366,8 +496,8 @@ class OneToManyManyToOneTest(AssertMixin):
 
         Ball.mapper = mapper(Ball, ball)
         Person.mapper = mapper(Person, person, properties= dict(
-         balls = relation(Ball.mapper, primaryjoin=ball.c.person_id==person.c.id, foreignkey=ball.c.person_id, private=True, post_update=True),
-         favorateBall = relation(Ball.mapper, primaryjoin=person.c.favorite_ball_id==ball.c.id, foreignkey=person.c.favorite_ball_id),
+         balls = relation(Ball.mapper, primaryjoin=ball.c.person_id==person.c.id, remote_side=ball.c.person_id, private=True, post_update=True, backref='person'),
+         favorateBall = relation(Ball.mapper, primaryjoin=person.c.favorite_ball_id==ball.c.id, remote_side=person.c.favorite_ball_id),
          )
         )
 
@@ -521,19 +651,19 @@ class SelfReferentialPostUpdateTest(AssertMixin):
                 primaryjoin=node_table.c.id==node_table.c.parent_id,
                 lazy=True,
                 cascade="all",
-                backref=backref("parent", primaryjoin=node_table.c.parent_id==node_table.c.id, foreignkey=node_table.c.id)
+                backref=backref("parent", primaryjoin=node_table.c.parent_id==node_table.c.id, remote_side=node_table.c.id)
             ),
             'prev_sibling': relation(
                 Node,
                 primaryjoin=node_table.c.prev_sibling_id==node_table.c.id,
-                foreignkey=node_table.c.id,
+                remote_side=node_table.c.id,
                 lazy=True,
                 uselist=False
             ),
             'next_sibling': relation(
                 Node,
                 primaryjoin=node_table.c.next_sibling_id==node_table.c.id,
-                foreignkey=node_table.c.id,
+                remote_side=node_table.c.id,
                 lazy=True,
                 uselist=False,
                 post_update=True
